"use strict";

var Promise = require("bluebird");

var logger = require('../logger');
var functions = require('../Functions');

var AbstractCustomMap = require('./AbstractCustomMap');
var AliasMarker = require('./marker/AliasMarker');
var CommentDialog = require('../gui/CommentDialog');
var ControlType = require('./ControlType');
var CustomMapOptions = require('./CustomMapOptions');
var IdentifiedElement = require('./data/IdentifiedElement');
var OverlayCollection = require('./overlay/OverlayCollection');
var PointMarker = require('./marker/PointMarker');
var ReactionMarker = require('./marker/ReactionMarker');
var ReactionOverlay = require('./overlay/ReactionOverlay');
var ReferenceGenome = require('./data/ReferenceGenome');
var Submap = require('./Submap');
var TouchMap = require('./TouchMap');

/**
 * Default constructor.
 * 
 * @param globalMap
 *          google.maps.Map object representing the map
 * @param configuration
 *          Configuration object representing our data in the map
 * @param bigButtons
 *          boolean value determining if the buttons on the map should be big,
 *          and if the map is run on the touch interface
 * @param hideDiv
 * 
 */
function CustomMap(options) {
  this._controls = [];
  
  if (!(options instanceof CustomMapOptions)) {
    options = new CustomMapOptions(options);
  }
  AbstractCustomMap.call(this, options.getProject().getModel(), options);
  this.setProject(options.getProject());

  // set config parameters
  this.setGoogleMap(options.getMap());

  if (options.isCustomTouchInterface()) {
    this._touchInterface = new TouchMap(this);
  }

  // create function that override primefaces fitBounds with default google
  // implementation
  var fitBounds = function(bounds) {
    var tmp = this.fitBounds;
    this.fitBounds = google.maps.Map.prototype.fitBounds;
    this.fitBounds(bounds);
    this.fitBounds = tmp;
  };
  this.getGoogleMap().fitBounds2 = fitBounds;

  this.buttons = [];

  this.createSubmaps();

  this.selectedLayouts = [];

  this.setupLayouts();

  this.createBelt();

  this.customizeGoogleMapView(options.getFitMapBounds());

  this.createMapChangedCallbacks();

  this.createClientServerListeners();

  this.overlayCollections = [];

  // which submap is active (where user made interaction for the last time)
  this._activeSubmapId = null;

  this.initialized = true;

  // list of reference genomes
  this._referenceGenome = [];
  
  var commentDialog = new CommentDialog(document.getElementById("feedbackContent"), this); 

  this.setCommentDialog(commentDialog);
  
  
  ServerConnector.actualizeSessionData();
}

CustomMap.prototype = Object.create(AbstractCustomMap.prototype);
CustomMap.prototype.constructor = CustomMap;

CustomMap.prototype.createSubmaps = function() {
  this.submaps = [];
  for (var i = 0; i < this.getModel().getSubmodels().length; i++) {
    this.submaps.push(new Submap(this, this.getModel().getSubmodels()[i]));
  }
};

CustomMap.prototype.createLogo = function() {

  var logoControlDiv2 = document.createElement('DIV');
  var logo2 = document.createElement('IMG');
  var url = ServerConnector.getLogoImg();
  if (!/^(f|ht)tps?:\/\//i.test(url)) {
    url = GuiConnector.getImgPrefix() + url;
  }
  logo2.src = url;
  logo2.style.cursor = 'pointer';
  logo2.style.width = "80px";
  logoControlDiv2.appendChild(logo2);
  google.maps.event.addDomListener(logo2, 'click', function() {
    var win = window.open(ServerConnector.getLogoLink(), '_blank');
    win.focus();
  });
  logoControlDiv2.index = 0; // used for ordering
  this.getGoogleMap().controls[google.maps.ControlPosition.LEFT_BOTTOM].push(logoControlDiv2);

  var logoControlDiv = document.createElement('DIV');
  logoControlDiv.style.padding = '5px';

  var logo = document.createElement('IMG');
  logo.src = GuiConnector.getImgPrefix() + GuiConnector.getLcsbLogoImg(this.bigButtons);
  logo.style.cursor = 'pointer';
  logoControlDiv.appendChild(logo);
  google.maps.event.addDomListener(logo, 'click', function() {
    var win = window.open('http://wwwen.uni.lu/lcsb/', '_blank');
    win.focus();
  });

  logoControlDiv.index = 1; // used for ordering
  this.getGoogleMap().controls[google.maps.ControlPosition.RIGHT_BOTTOM].push(logoControlDiv);
};

CustomMap.prototype.createBelt = function() {
  var self = this;

  this.divBelt = document.createElement('DIV');
  this.divBelt.className = "headerBelt";

  var hideDivButton = document.createElement('DIV');
  hideDivButton.className = "headerHideDivButton";

  var hideButton = document.createElement('button');
  hideButton.id = "hide_button";
  hideButton.className = "headerHideButton";
  hideButton.innerHTML = "<i class='fa fa-chevron-left'></i>";
  // when there is no div to hide we should allow hiding
  if (self.getHideDiv() !== undefined) {
    hideButton.onclick = (function() {
      var button = hideButton;
      var div = self.getHideDiv();

      var left = GuiConnector.getObjectByPrimefaceId(self.getGoogleMap().getDiv().id).offset().left;
      return function() {
        if (button.innerHTML.indexOf('fa-chevron-left') > 0) {
          button.innerHTML = "<i class='fa fa-chevron-right'></i>";
          div.style.display = 'none';
          self.getGoogleMap().getDiv().style.left = "0px";
        } else {
          div.style.display = 'block';
          button.innerHTML = "<i class='fa fa-chevron-left'></i>";
          self.getGoogleMap().getDiv().style.left = left + "px";
        }
        google.maps.event.trigger(self.getGoogleMap(), 'resize');
        return false;
      };
    })();
  } else {
    hideButton.disabled = true;
    logger.warn("Left panel hiding disabled");
  }
  hideDivButton.appendChild(hideButton);
  hideDivButton.index = 1; // used for ordering
  this.divBelt.appendChild(hideDivButton);

  var controlText = document.createElement('div');
  controlText.className = "headerTextBold";
  controlText.innerHTML = this.getProject().getName();
  this.divBelt.appendChild(controlText);

  this.getGoogleMap().controls[google.maps.ControlPosition.TOP_LEFT].push(this.divBelt);
};

CustomMap.prototype.setLegendVisible = function(vis) {
  if (vis) {
    document.getElementById('legend').style.display = 'block';
  } else {
    document.getElementById('legend').style.display = 'none';
  }
};

CustomMap.prototype.clearOverlays = function() {
  for ( var overlayName in this.overlayCollections) {
    if (this.overlayCollections.hasOwnProperty(overlayName)) {
      var collection = this.overlayCollections[overlayName];
      this.clearOverlayCollection(collection);
    }
  }
};

CustomMap.prototype.refreshOverlays = function() {
  var promises = [];
  for ( var overlayName in this.overlayCollections) {
    if (this.overlayCollections.hasOwnProperty(overlayName)) {
      var collection = this.overlayCollections[overlayName];
      promises.push(collection.refresh());
    }
  }
  return Promise.all(promises);
};

/**
 * Removes all markers from {@link OverlayCollection}.
 * 
 * @param collection
 *          {@link OverlayCollection} from which all markers should be removed
 */
CustomMap.prototype.clearOverlayCollection = function(collection) {
  logger.debug("Clear collection: " + collection.name);
  for ( var aliasKey in collection.aliasMarkers) {
    if (collection.aliasMarkers.hasOwnProperty(aliasKey) && collection.aliasMarkers[aliasKey] !== undefined
        && collection.aliasMarkers[aliasKey] !== null) {
      collection.aliasMarkers[aliasKey].setMap(null);
    }
  }

  for ( var pointKey in collection.pointMarkers) {
    if (collection.pointMarkers.hasOwnProperty(pointKey) && collection.pointMarkers[pointKey] !== undefined
        && collection.pointMarkers[pointKey] !== null) {
      collection.pointMarkers[pointKey].setMap(null);
    }
  }

  for ( var key in collection.reactionMarkers) {
    if (collection.reactionMarkers.hasOwnProperty(key) && collection.reactionMarkers[key] !== undefined
        && collection.reactionMarkers[key] !== null) {
      collection.reactionMarkers[key].setMap(null);
    }
  }

  collection.aliasMarkers = [];
  collection.pointMarkers = [];
  collection.reactionMarkers = [];
};

/**
 * Updates data about visualized markers in {@link OverlayCollection}.
 * 
 * @param overlayCollection
 *          {@link OverlayCollection} with new data to visualize
 * @param fitBounds
 *          <code>true</code> id the map should fit bounds to the new elements
 *          after update, <code>false</code> otherwise
 */
CustomMap.prototype.updateOverlayCollection = function(overlayCollection, fitBounds) {
  this.clearOverlayCollection(overlayCollection);
  return this.renderOverlayCollection({overlayCollection:overlayCollection, fitBounds:fitBounds});
};

/**
 * This method open layout by a given layout identifier (string starting with
 * 'cv' prefix) in a map and all submaps.
 * 
 * @param identifier
 *          identifier of the layout to present
 */
CustomMap.prototype.openLayout = function(identifier) {
  logger.debug("Opening layout: " + identifier);

  this.getGoogleMap().setMapTypeId(identifier);

  var index = null;
  for (var j = 0; j < this.getLayouts().length; j++) {
    var layout = this.getLayouts()[j];
    if ('cv' + layout.getId() === identifier) {
      index = j;
    }
  }
  if (index === null) {
    logger.warn("Invalid layout identifier: " + identifier);
  }
  for (var i = 0; i < this.submaps.length; i++) {
    this.submaps[i].openLayout('cv' + this.submaps[i].getModel().getLayouts()[index].getId());
  }
};

/**
 * This method open layout by a given database identifier.
 * 
 * @param identifier
 *          identifier of the layout to present
 */
CustomMap.prototype.openLayoutById = function(identifier) {
  identifier = parseInt(identifier);
  logger.debug("Opening layout: " + identifier);
  var index = null;
  for (var i = 0; i < this.getLayouts().length; i++) {
    var layout = this.getLayouts()[i];
    if (layout.getId() === identifier) {
      index = 'cv' + identifier;
    }
  }

  // if layout doesn't exist print error
  if (index === null) {
    throw new Error("You have no privileges for selected layout");
  } else {
    this.openLayout(index);
  }
};

CustomMap.prototype.openLayoutByName = function(name) {
  logger.debug("Opening layout: " + name);
  var index = null;
  var layouts = this.getLayouts();
  for (var i = 0; i < layouts.length; i++) {
    var layout = layouts[i];
    if (layout.getName() === name) {
      index = 'cv' + layout.getId();
    }
  }
  // if layout doesn't exist print error
  if (index === null) {
    alert("You have no privileges for selected layout");
  } else {
    this.openLayout(index);
  }
};

CustomMap.prototype.createMapMenu = function() {
  var selfMap = this;

  var buttons = [];

  // create a button for overview images when the image is available
  if (this.getTopOverviewImage() !== undefined && this.getTopOverviewImage() !== null) {
    var submenuButtonDiv2 = document.createElement('button');
    buttons.push(submenuButtonDiv2);
    submenuButtonDiv2.id = "overview_button";
    submenuButtonDiv2.innerHTML = "<i class='fa fa-sitemap' style='font-size:18px; font-weight:400; padding-right:10px;'></i> SHOW OVERVIEW";
    submenuButtonDiv2.className = "overview_button";
    submenuButtonDiv2.onclick = (function() {
      return function() {
        selfMap.showOverview();
        return false;
      };
    })();
    this.divBelt.appendChild(submenuButtonDiv2);
  }

  var rightHeaderMenu = document.createElement('div');
  rightHeaderMenu.className = "rightHeaderMenu";
  var submenuDiv = document.createElement('div');
  submenuDiv.className = "div4checkboxes";
  var commentCheckbox = document.createElement('input');
  commentCheckbox.type = "checkbox";
  commentCheckbox.name = "Comments";
  commentCheckbox.id = ControlType.COMMENT_CHECKBOX;
  commentCheckbox.onclick = (function() {
    var selfButton = commentCheckbox;
    return function() {
      ServerConnector.getSessionData().setShowComments(selfButton.checked);
      if (selfButton.checked) {
        document.getElementById('refresh_comments_button').style.display = 'inline';
      } else {
        document.getElementById('refresh_comments_button').style.display = 'none';
      }
      selfMap.refreshComments();
    };
  })();
  this.addControl(commentCheckbox);
  var element = document.createElement('label');
  element.innerHTML = "COMMENTS";
  element.setAttribute("for", ControlType.COMMENT_CHECKBOX);
  submenuDiv.appendChild(commentCheckbox);
  submenuDiv.appendChild(element);

  var submenuButtonDiv = document.createElement('input');
  submenuButtonDiv.type = "checkbox";
  submenuButtonDiv.name = "Legend";
  submenuButtonDiv.id = "lengend_checkbox";
  submenuButtonDiv.onclick = (function() {
    var selfButton = submenuButtonDiv;
    return function() {
      if (selfButton.checked) {
        GuiConnector.showLegend();
      } else {
        GuiConnector.hideLegend();
      }
    };
  })();
  element = document.createElement('label');
  element.innerHTML = "LEGEND";
  element.setAttribute("for", "lengend_checkbox");
  submenuDiv.appendChild(submenuButtonDiv);
  submenuDiv.appendChild(element);

  submenuButtonDiv = document.createElement('button');
  submenuButtonDiv.id = "refresh_comments_button";
  submenuButtonDiv.innerHTML = "<i class='fa fa-refresh' style='font-size:21px; font-weight:400;'></i>";
  submenuButtonDiv.className = "overview_button";
  submenuButtonDiv.style.display = 'none';
  submenuButtonDiv.onclick = (function() {
    return function() {
      selfMap.refreshComments();
      return false;
    };
  })();
  submenuDiv.appendChild(submenuButtonDiv);
  rightHeaderMenu.appendChild(submenuDiv);

  submenuButtonDiv = document.createElement('button');
  buttons.push(submenuButtonDiv);
  submenuButtonDiv.id = "clear_button";
  submenuButtonDiv.className = "overview_button";
  submenuButtonDiv.innerHTML = "<i class='fa fa-times' style='font-size:18px; font-weight:300; padding-right:10px;'></i> CLEAR";
  submenuButtonDiv.title = "Clear all queries";
  submenuButtonDiv.style.display = 'inline';
  submenuButtonDiv.onclick = (function() {
    return function() {
      selfMap.clearData();
      return false;
    };
  })();
  rightHeaderMenu.appendChild(submenuButtonDiv);

  this.divBelt.appendChild(rightHeaderMenu);
};

CustomMap.prototype.registerSource = function(overlayCollection) {
  this.overlayCollections[overlayCollection.name] = overlayCollection;
  overlayCollection.aliasMarkers = [];
  overlayCollection.pointMarkers = [];
  overlayCollection.reactionMarkers = [];
};

CustomMap.prototype.refreshComments = function() {
  var self = this;
  for ( var overlayName in this.overlayCollections) {
    if (this.overlayCollections.hasOwnProperty(overlayName) && overlayName === "comment") {
      var collection = this.overlayCollections[overlayName];
      self.clearOverlayCollection(collection);
      if (ServerConnector.getSessionData().getShowComments()) {
        return collection.refresh().then(function(){
          return self.renderOverlayCollection({overlayCollection:collection});        
        });
      } else {
        return collection.clear();
      }
    }
  }
  throw new Error("comment OverlayCollection not found");
};

CustomMap.prototype.turnOnOffDrawing = function() {
  var model = this.getSubmodelById(this.getActiveSubmapId());
  if (model !== undefined) {
    model._turnOnOffDrawing();
  } else {
    throw "Cannot find submodel with id: " + this.getActiveSubmapId();
  }
};

CustomMap.prototype.clearData = function() {
  this.clearOverlays();
  for ( var overlayName in this.overlayCollections) {
    if (this.overlayCollections.hasOwnProperty(overlayName)) {
      ServerConnector.sendClearRequest(overlayName);
    }
  }
};

CustomMap.prototype.refreshMarkers = function() {
  logger.debug("Refresh Markers: ");
  for ( var overlayName in this.overlayCollections) {
    if (this.overlayCollections.hasOwnProperty(overlayName)) {
      var collection = this.overlayCollections[overlayName];
      this.refreshOverlayMarkers(collection);
    }
  }
};

CustomMap.prototype.refreshOverlayMarkers = function(overlay) {
  logger.debug("Refresh overlay: " + overlay.name);
  var boundsArray = [];
  boundsArray[this.getId()] = new google.maps.LatLngBounds();
  for (var i = 0; i < this.submaps.length; i++) {
    boundsArray[this.submaps[i].getId()] = new google.maps.LatLngBounds();
  }

  var updated = false;
  var stillMissing = false;

  var bounds;
  for ( var aliasKey in overlay.aliasMarkers) {
    if (overlay.aliasMarkers.hasOwnProperty(aliasKey) && overlay.aliasMarkers[aliasKey] !== undefined
        && overlay.aliasMarkers[aliasKey] !== null) {
      var alias = overlay.aliasMarkers[aliasKey];
      if (alias.getAliasData() === null || alias.getAliasData() === undefined) {
        alias.getCustomMap().getModel().getAliasById(alias.getId()).then(function(aliasData) {
          alias.setAliasData(aliasData);
          alias.init();
          alias.show();
          updated = true;
          bounds = alias.getBounds();
          boundsArray[alias.getCustomMap().getId()].extend(bounds.getNorthEast());
        });
      } else {
        bounds = alias.getBounds();
        if (!this.isMarkerOptimization()) {
          alias.hide();
          alias.show();
        }
        boundsArray[alias.getCustomMap().getId()].extend(bounds.getNorthEast());
        boundsArray[alias.getCustomMap().getId()].extend(bounds.getSouthWest());
      }
    }
  }

  for ( var key in overlay.pointMarkers) {
    if (overlay.pointMarkers.hasOwnProperty(key) && overlay.pointMarkers[key] !== undefined
        && overlay.pointMarkers[key] !== null) {
      var pointMarker = overlay.pointMarkers[key];
      // we don't need to update this markers because thet data about
      // visualization is
      // already there
      // alias.update();
      bounds = pointMarker.getBounds();
      if (!this.isMarkerOptimization()) {
        pointMarker.hide();
        pointMarker.show();
      }
      boundsArray[pointMarker.getCustomMap().getId()].extend(bounds.getNorthEast());
    }
  }

  for ( var reactionKey in overlay.reactionMarkers) {
    if (overlay.reactionMarkers.hasOwnProperty(reactionKey) && overlay.reactionMarkers[reactionKey] !== undefined
        && overlay.reactionMarkers[reactionKey] !== null) {
      var reactionOverlay = overlay.reactionMarkers[reactionKey];
      if (reactionOverlay.getReactionData() === null || reactionOverlay.getReactionData() === undefined) {
        reactionOverlay.getCustomMap().getModel().getReactionById(reactionOverlay.getId()).then(function(reactionData) {
          reactionOverlay.setReactionData(reactionData);
          reactionOverlay.init();
          reactionOverlay.show();
          updated = true;
          bounds = reactionOverlay.getBounds();
          boundsArray[reactionOverlay.getCustomMap().getId()].extend(bounds.getNorthEast());
          boundsArray[reactionOverlay.getCustomMap().getId()].extend(bounds.getSouthWest());
        });
      } else {
        bounds = reactionOverlay.getBounds();
        if (!this.isMarkerOptimization()) {
          reactionOverlay.hide();
          reactionOverlay.show();
        }
        boundsArray[reactionOverlay.getCustomMap().getId()].extend(bounds.getNorthEast());
        boundsArray[reactionOverlay.getCustomMap().getId()].extend(bounds.getSouthWest());
      }
    }
  }

  if (!stillMissing && updated && overlay.fitBounds) {
    for ( var mapId in boundsArray) {
      if (boundsArray.hasOwnProperty(mapId)) {
        var map = this.getSubmodelById(mapId).getGoogleMap();
        bounds = boundsArray[mapId];
        if (map !== undefined && !bounds.isEmpty()) {
          if (typeof map.fitBounds2 !== "undefined") {
            map.fitBounds2(bounds);
          } else {
            map.fitBounds(bounds);
          }
        }
      }
    }
  }
};

CustomMap.prototype.openSubmodel = function(id, htmlTag, jsVar) {
  if (jsVar.submapControler === undefined) {
    var submap = null;
    for (var j = 0; j < this.submaps.length; j++) {
      if (this.submaps[j].getId() === id) {
        submap = this.submaps[j];
      }
    }
    if (submap === null) {
      throw new Error("Unknown submap for id: " + id);
    } else {
      submap.init(htmlTag, jsVar);
      // we have to perform it on top map, because on submaps id is different
      this.openLayout(this.getGoogleMap().getMapTypeId());

      this.refreshOverlays();

      // now we have to visualize layouts
      var layouts = [];

      // get list of layouts
      for ( var key in this.selectedLayouts) {
        if (this.selectedLayouts.hasOwnProperty(key) && this.selectedLayouts[key] === true) {
          layouts.push(key);
        }
      }

      // show layouts that should be visualized (resize or show them)
      for (var i = 0; i < layouts.length; i++) {
        var layoutId = layouts[i];
        submap._showSelectedLayout(layoutId, i, layouts.length);
      }
    }
  }
  jsVar.show();

};

CustomMap.prototype.customizeGoogleMapView = function(fitMapBounds) {
  var mapOptions = this.creatMapOptions();
  this.getGoogleMap().setOptions(mapOptions);

  this.createMapMenu();

  this.registerMapClickEvents();

  this.createLogo();
  // this.createMapVersion();
  google.maps.event.trigger(this.getGoogleMap(), 'resize');
  google.maps.event.trigger(this.getGoogleMap(), 'maptypeid_changed');
  google.maps.event.trigger(this.getGoogleMap(), 'projection_changed');

  // center map and zoom in to fit into browser window
  if (fitMapBounds) {
    var bounds = new google.maps.LatLngBounds();
    bounds.extend(this.getTopLeftLatLng());
    bounds.extend(this.getBottomRightLatLng());

    if (typeof this.getGoogleMap().fitBounds2 !== undefined) {
      this.getGoogleMap().fitBounds2(bounds);
    } else {
      this.getGoogleMap().fitBounds(bounds);
    }
  }
};

CustomMap.prototype.setCenter = function(mapIdentifier, coordinates) {
  if (this.getModel().getId() === mapIdentifier) {
    this.getGoogleMap().setCenter(coordinates);
  } else {
    GuiConnector.openDialog(mapIdentifier);
    for (var i = 0; i < this.submaps.length; i++) {
      if (this.submaps[i].getId() === mapIdentifier) {
        if (coordinates instanceof google.maps.Point) {
          coordinates = this.submaps[i].fromPointToLatLng(coordinates);
        }
        this.submaps[i].getGoogleMap().setCenter(coordinates);
      }
    }
  }
};

CustomMap.prototype.setZoom = function(mapIdentifier, zoom) {
  if (this.getModel().getId() === mapIdentifier) {
    this.getGoogleMap().setZoom(zoom);
  } else {
    GuiConnector.openDialog(mapIdentifier);
    for (var i = 0; i < this.submaps.length; i++) {
      if (this.submaps[i].getId() === mapIdentifier) {
        this.submaps[i].getGoogleMap().setZoom(zoom);
      }
    }
  }
};

/**
 * Creates listeners for google.maps.Map object that will actualize the data in
 * user session.
 */
CustomMap.prototype.createMapChangedCallbacks = function() {
  var customMapSelf = this;
  // listener for changing zoom level
  google.maps.event.addListener(this.getGoogleMap(), 'zoom_changed', function() {
    ServerConnector.setZoomLevel(customMapSelf.getGoogleMap().getZoom());
    ServerConnector.actualizeSessionData();
  });

  // if we have zoom level data stored in session then restore it
  var level = ServerConnector.getZoomLevel();
  if (parseInt(level) > 0) {
    level = parseInt(level);
    this.getGoogleMap().setZoom(level);
  } else {
    ServerConnector.setZoomLevel(customMapSelf.getGoogleMap().getZoom());
  }

  // listener for changing location of the map (moving left/reght/top/bottom
  google.maps.event.addListener(this.getGoogleMap(), 'center_changed', function() {
    var coord = customMapSelf.getGoogleMap().getCenter();
    var point = customMapSelf.fromLatLngToPoint(coord);
    ServerConnector.setCenterCoordinateX(point.x);
    ServerConnector.setCenterCoordinateY(point.y);
    ServerConnector.actualizeSessionData();
  });

  // if we have coordinate data stored in session then restore it
  var x = ServerConnector.getCenterCoordinateX();
  var y = ServerConnector.getCenterCoordinateY();
  if (!isNaN(x) && !isNaN(y)) {
    var point = new google.maps.Point(x, y);
    var coord = customMapSelf.fromPointToLatLng(point);
    customMapSelf.getGoogleMap().setCenter(coord);
  }

  // listener for changing type of layout
  google.maps.event.addListener(this.getGoogleMap(), 'maptypeid_changed', function() {
    ServerConnector.setSelectedLayout(customMapSelf.getGoogleMap().getMapTypeId());
    ServerConnector.actualizeParams();
  });

  // if we have type of layout stored in the session then restore it
  var mapType = ServerConnector.getSelectedLayout();
  if (mapType !== "" && mapType !== null && mapType !== undefined) {
    this.openLayout(mapType);
  }
};

/**
 * Returns submodel (or this model) by identfier of the model.
 * 
 * @param identifier
 *          identifier of the submodel
 * @returns submodel (or this model) with given identfier of the model
 */
CustomMap.prototype.getSubmodelById = function(identifier) {
  identifier = parseInt(identifier);
  if (this.getId() === identifier) {
    return this;
  }
  for (var i = 0; i < this.submaps.length; i++) {
    if (this.submaps[i].getId() === identifier) {
      return this.submaps[i];
    }
  }
  return null;
};

CustomMap.prototype.removeSelection = function() {
  var model = this.getSubmodelById(this.getActiveSubmapId());
  if (model !== null) {
    model._removeSelection();
  } else {
    throw "Cannot find submodel with id: " + this.getActiveSubmapId();
  }
};

/**
 * This method will hide google map view and will present single image overview
 * of the data.
 */
CustomMap.prototype.showOverview = function(overviewImageId) {
  var overviewDialog = GuiConnector.getOverviewDialog();
  overviewDialog.syncWindowResize();
  if (this.getOverviewDiv() === undefined) {
    logger.warn("Cannot show overview, because overview div is  undefined");
  } else {
    logger.debug("Show overview");
    overviewDialog.show();

    // resize dialog
    var htmlTag = GuiConnector.getOverviewHtmlTag();

    var width = Math.floor(window.innerWidth * 2 / 3);
    var height = Math.floor(window.innerHeight * 2 / 3);

    htmlTag.style.height = (height + 50) + "px";
    htmlTag.style.width = (width + 20) + "px";

    var self = this;

    // remove all child nodes from overview div
    while (this.getOverviewDiv().hasChildNodes()) {
      this.getOverviewDiv().removeChild(this.getOverviewDiv().lastChild);
    }

    if (overviewImageId === undefined || overviewImageId === null) {
      this.overviewImage = this.getConfiguration().TOP_OVERVIEW_IMAGE;
    } else {
      this.overviewImage = null;
      for (var i = 0; i < this.getConfiguration().OVERVIEW_IMAGES.length; i++) {
        if (this.getConfiguration().OVERVIEW_IMAGES[i].idObject === overviewImageId) {
          this.overviewImage = this.getConfiguration().OVERVIEW_IMAGES[i];
        }
      }

      if (this.overviewImage === null) {
        logger.warn("Unknown overview image with id = " + overviewImageId);
        this.overviewImage = this.getConfiguration().TOP_OVERVIEW_IMAGE;
      }
    }

    // add image to overview div
    this.overviewImageTag = document.createElement("IMG");
    this.overviewImageTag.src = "../map_images/" + this.overviewImage.filename;
    this.getOverviewDiv().appendChild(this.overviewImageTag);

    var ratio = 1.0;

    // check how image should be resized to fit dialog and resize it manually!!!
    if (width / this.overviewImage.width > height / this.overviewImage.height) {
      this.overviewImageTag.style.height = height + "px";
      ratio = height / this.overviewImage.height;
      width = this.overviewImage.width * ratio;

      htmlTag.style.width = (width + 20) + "px";
    } else {
      this.overviewImageTag.style.width = width + "px";
      ratio = width / this.overviewImage.width;
      height = this.overviewImage.height * ratio;

      htmlTag.style.height = (height + 50) + "px";
    }

    // center dialog
    overviewDialog.jq.css("top", Math.max(0, (($(window).height() - overviewDialog.jq.outerHeight()) / 2)
        + $(window).scrollTop())
        + "px");
    overviewDialog.jq.css("left", Math.max(0, (($(window).width() - overviewDialog.jq.outerWidth()) / 2)
        + $(window).scrollLeft())
        + "px");

    // on click event (what should happen when we click on the image)
    var onclickevent = function getClickPosition(e) {
      var parentPosition = functions.getPosition(e.currentTarget);
      var xPosition = e.clientX - parentPosition.x;
      var yPosition = e.clientY - parentPosition.y;

      var imgWidth = self.overviewImageTag.offsetWidth;

      var currentRatio = imgWidth / self.overviewImage.width;

      var xNormal = xPosition / currentRatio;
      var yNormal = yPosition / currentRatio;
      var point = {
        x : xNormal,
        y : yNormal
      };

      var link = null;
      for (var i = 0; i < self.overviewImage.links.length; i++) {
        if (functions.pointInsidePolygon(point, self.overviewImage.links[i].polygon)) {
          if (link === null) {
            link = self.overviewImage.links[i];
          } else {
            logger.warn("More than one link found. Skipping");
          }
        }
      }
      if (link !== null) {
        if (link.type === "OverviewModelLink") {
          logger.debug("Opening model from overview. ModelId: " + link.modelLinkId);
          logger.debug("link coordinates [" + link.idObject + "]: " + link.latLng);
          // TODO min zoom value can be different for every map, it should be
          // changed in the future
          self.showModel(link.modelLinkId, link.latLng, link.zoomLevel + self.getConfiguration().MIN_ZOOM);
          overviewDialog.hide();
        } else if (link.type === "OverviewImageLink") {
          logger.debug("Opening image from overview. ImageId: " + link.imageLinkId);
          self.showOverview(link.imageLinkId);
        } else if (link.type === "OverviewSearchLink") {
          logger.debug("Sending search query. Query: " + link.query);
          GuiConnector.search(link.query);
          overviewDialog.hide();
        } else {
          logger.warn("Unknown type of link: " + link.type + ". Don't know what to do... LinkId: " + link.idObject);
        }
      }
    };

    this.overviewImageTag.onclick = onclickevent;

    // resize canvas where on mouse over highligh will appear
    var canvas = document.getElementById("canvasDebug");
    canvas.width = width;
    canvas.height = height;
    canvas.onclick = onclickevent;

    // in debug mode draw clickable shapes
    if (this.isDebug()) {
      this.drawClickableShapes(canvas, ratio);
    }

    this.overviewImage.mousePos = {
      x : 0,
      y : 0
    };

    // this listener should be called when mouse moves over image, it purpose is
    // to change coursor to pointer when mouse enters clickable polygon and back
    // to normal when mouse leaves such region
    var onmousemove = function getMouseOverPosition(e) {
      var position = functions.getPosition(e.currentTarget);
      position.x = e.clientX - position.x;
      position.y = e.clientY - position.y;

      var imgWidth = self.overviewImageTag.offsetWidth;

      var currentRatio = imgWidth / self.overviewImage.width;

      var xNormal = position.x / currentRatio;
      var yNormal = position.y / currentRatio;
      var point = {
        x : xNormal,
        y : yNormal
      };

      if (self.overviewImage.mousePos.x !== position.x || self.overviewImage.mousePos.y !== position.y) {
        self.overviewImage.mousePos = position;
        var link = null;
        for (var i = 0; i < self.overviewImage.links.length; i++) {
          if (functions.pointInsidePolygon(point, self.overviewImage.links[i].polygon)) {
            link = self.overviewImage.links[i];
          }
        }
        if (link === null) {
          e.currentTarget.style.cursor = "auto";
        } else {
          e.currentTarget.style.cursor = "pointer";
        }
      }
    };

    // onmousemove listener should be assigned to canvas (which is on top of the
    // image) and overviewimage (just in case something went wrong with resizing
    // canvas)
    canvas.onmousemove = onmousemove;
    this.overviewImageTag.onmousemove = onmousemove;
  }
};

CustomMap.prototype.drawClickableShapes = function(canvas, ratio) {
  var ctx = canvas.getContext("2d");
  // clear canvas
  ctx.clearRect(0, 0, canvas.width, canvas.height);
  for (var i = 0; i < this.overviewImage.links.length; i++) {
    ctx.beginPath();
    var polygon = this.overviewImage.links[i].polygon;
    for (var j = 0; j < polygon.length; j++) {
      var x = polygon[j].x * ratio;
      var y = polygon[j].y * ratio;
      ctx.moveTo(x, y);
      x = polygon[(j + 1) % polygon.length].x * ratio;
      y = polygon[(j + 1) % polygon.length].y * ratio;
      ctx.lineTo(x, y);
    }
    ctx.stroke();
  }
};

CustomMap.prototype.showModel = function(id, point, zoomLevel) {
  if (point !== undefined) {
    this.setCenter(id, point);
  } else {
    logger.warn("Center point undefined...");
  }
  if (zoomLevel !== undefined) {
    this.setZoom(id, zoomLevel);
  } else {
    logger.warn("Zoom level undefined...");
  }
};

/**
 * Adds information about aliases visible in the specific layout.
 * 
 * @param layoutId
 *          identifier of the layout
 * 
 * @param jsonAliases
 *          list of aliases in json format
 * 
 */
CustomMap.prototype.addAliasesForLayout = function(layoutId, jsonAliases) {
  logger.debug("Adding aliases for layout: " + layoutId);

  // init layout data
  if (this.getModel().getLayoutDataById(layoutId) === undefined) {
    this.getModel().initLayoutData(layoutId);
    for (var j = 0; j < this.submaps.length; j++) {
      this.submaps[j].getModel().initLayoutData(layoutId);
    }
  }

  var aliases = JSON.parse(jsonAliases);
  for (var i = 0; i < aliases.length; i++) {
    var alias = aliases[i];
    var model = this.getSubmodelById(alias.modelId);
    if (model !== null) {
      model.getModel().addAliasForLayout(layoutId, alias);
    } else {
      logger.warn("Unknown model: " + alias.modelId);
    }
  }

  this.retrieveMissingAliases();
};

/**
 * Adds information about aliases.
 * 
 * @param jsonAliases
 *          list of aliases in json format
 * 
 */
CustomMap.prototype.addAliases = function(aliases) {
  for (var i = 0; i < aliases.length; i++) {
    var alias = aliases[i];
    var model = this.getSubmodelById(alias.modelId);
    if (model !== undefined) {
      model.addAlias(alias);
    } else {
      logger.warn("Unknown model: " + alias.modelId);
    }
  }
  this.callListeners("onAddAliases");
};

/**
 * This function will ask server for aliases that should be visualized but the
 * data is still missing on the client side.
 */
CustomMap.prototype.retrieveMissingAliases = function() {
  var ids = [];
  var missing = this.getModel().getMissingAliasIds();
  for (var k = 0; k < missing.length; k++) {
    ids.push([ this.getId(), missing[k] ]);
  }
  for (var i = 0; i < this.submaps.length; i++) {
    missing = this.submaps[i].getModel().getMissingAliasIds();
    for (var j = 0; j < missing.length; j++) {
      ids.push([ this.submaps[i].getId(), missing[j] ]);
    }
  }
  if (ids.length > 0) {
    // load data from server about missing aliases
    ServerConnector.retreiveLightAliases(ids);
  }
  if (!ServerConnector.isWaitingForData()) {
    // if we already have everything then just refresh data to be visualized
    this.refreshSelectedLayouts();
    // and close "loading" dialog
    GuiConnector.closeLoadingDialog();
  }
};

/**
 * Adds layout to be visualized.
 * 
 * @param identifier
 *          identifier of the layout that should be included in visualization
 */
CustomMap.prototype.addSelectedLayout = function(identifier) {
  var self = this;
  return new Promise(function(resolve, reject) {
    logger.debug("Selecting layout: " + identifier);

    if (self.selectedLayouts[identifier] === true) {
      logger.warn("Layout " + identifier + " already selected");
      resolve();
    } else {
      self.selectedLayouts[identifier] = true;
      return self.getModel().initLayoutDataByLayoutId(identifier).then(function() {
        return self.refreshSelectedLayouts();
      }).then(function() {
        return ServerConnector.setVisibleLayouts(JSON.stringify(self.getSelectedLayouts()));
      }).then(function() {
        resolve();
      }).catch(function(exception){
        reject(exception);
      });
    }
  });
};

/**
 * Returns list of layouts that are selected and visualized by javascript.
 * 
 * @return array with a list of selected layotus
 * 
 */
CustomMap.prototype.getSelectedLayouts = function() {
  var layouts = [];

  // get list of layouts
  for ( var key in this.selectedLayouts) {
    if (this.selectedLayouts.hasOwnProperty(key) && this.selectedLayouts[key] === true) {
      layouts.push(key);
    }
  }
  return layouts;
};

/**
 * Removes layout from visualization.
 * 
 * @param identifier
 *          identifier of layout to remove
 * 
 */
CustomMap.prototype.removeSelectedLayout = function(identifier) {
  logger.debug("Removing layout: " + identifier);

  if (this.selectedLayouts[identifier] !== true) {
    logger.warn("Layout " + identifier + " is not selected");
  } else {
    this.selectedLayouts[identifier] = false;
    this.refreshSelectedLayouts();
    ServerConnector.setVisibleLayouts(JSON.stringify(this.getSelectedLayouts()));
  }
};

/**
 * Refresh visualization of selected layouts.
 */
CustomMap.prototype.refreshSelectedLayouts = function() {
  logger.debug("Refreshing layouts");
  var self = this;
  var layouts = self.getSelectedLayouts();
  // show layouts that should be visualized (resize or show them)
  var promises = [];
  for (var i = 0; i < layouts.length; i++) {
    var layoutId = layouts[i];
    if (self.layoutContainsOverlays(layoutId)) {
      // resize element on the map
      promises.push(self.resizeSelectedLayout(layoutId, i, layouts.length));
    } else {
      promises.push(self.showSelectedLayout(layoutId, i, layouts.length));
    }
  }
  return Promise.all(promises).then(function() {
    // remove layouts that were
    for ( var key in self.selectedLayoutOverlays) {
      if (!self.selectedLayouts.hasOwnProperty(key) || self.selectedLayouts[key] === false) {
        if (self.layoutContainsOverlays(key)) {
          self.hideSelectedLayout(key);
        }
      }
    }
    self.refreshInfoWindows();    

  });
};

/**
 * Hides layout from the map and all submaps
 * 
 * @param layoutId
 *          identifier of a layout to hide
 */
CustomMap.prototype.hideSelectedLayout = function(layoutId) {
  this._hideSelectedLayout(layoutId);
  for (var i = 0; i < this.submaps.length; i++) {
    this.submaps[i]._hideSelectedLayout(layoutId);
  }
};

/**
 * Resize(refresh) layout on the map and all submaps. Resizing should be called
 * when number of layouts to visualize change.
 * 
 * @param layoutId
 *          identifier of layout to refresh
 * @param index
 *          position of the layout in list of layouts that we visualize
 * @param length
 *          number of layouts that we currently visualize
 */
CustomMap.prototype.resizeSelectedLayout = function(layoutId, index, length) {
  logger.debug("Resize layout: " + layoutId);
  var promises = [];
  promises.push(this._resizeSelectedLayout(layoutId, index, length));
  for (var i = 0; i < this.submaps.length; i++) {
    promises.push(this.submaps[i]._resizeSelectedLayout(layoutId, index, length));
  }
  return Promise.all(promises);
};

/**
 * Show layout on the map and all submaps.
 * 
 * @param layoutId
 *          identifier of layout to show
 * @param index
 *          position of the layout in list of layouts that we visualize
 * @param length
 *          number of layouts that we currently visualize
 */
CustomMap.prototype.showSelectedLayout = function(layoutId, index, length) {
  logger.debug("Resize layout: " + layoutId);
  var promises = [];
  promises.push(this._showSelectedLayout(layoutId, index, length));

  for (var i = 0; i < this.submaps.length; i++) {
    promises.push(this.submaps[i]._showSelectedLayout(layoutId, index, length));
  }
  return Promise.all(promises);
};

/**
 * Adds information about reactions visible in the specific layout.
 * 
 * @param layoutId
 *          identifier of the layout
 * 
 * @param jsonAliases
 *          list of reactions in json format
 * 
 */
CustomMap.prototype.addReactionsForLayout = function(layoutId, jsonReactions) {
  logger.debug("Adding reactions for layout: " + layoutId);
  var reactions = JSON.parse(jsonReactions);
  for (var i = 0; i < reactions.length; i++) {
    var reaction = reactions[i];
    var model = this.getSubmodelById(reaction.modelId);
    if (model !== undefined) {
      model.getModel().addReactionForLayout(layoutId, reaction);
    } else {
      logger.warn("Unknown model: " + reaction.modelId);
    }
  }
  this.retrieveMissingReactions();
};

/**
 * Adds information about reactions.
 * 
 * @param jsonAliases
 *          list of reactions in json format
 * 
 */
CustomMap.prototype.addReactions = function(jsonReactions) {
  var reactions = JSON.parse(jsonReactions);
  for (var i = 0; i < reactions.length; i++) {
    var reaction = reactions[i];
    var model = this.getSubmodelById(reaction.modelId);
    if (model !== undefined) {
      model.addReaction(reaction);
    } else {
      logger.warn("Unknown model: " + reaction.modelId);
    }
  }
  this.callListeners("onAddReactions");
};

/**
 * This function will ask server for reactions that should be visualized but the
 * data is still missing on the client side.
 */
CustomMap.prototype.retrieveMissingReactions = function() {
  var ids = [];
  var missing = this.getModel().getMissingReactionIds();
  for (var k = 0; k < missing.length; k++) {
    ids.push([ this.getId(), missing[k] ]);
  }
  for (var i = 0; i < this.submaps.length; i++) {
    missing = this.submaps[i].getModel().getMissingReactionIds();
    for (var j = 0; j < missing.length; j++) {
      ids.push([ this.submaps[i].getId(), missing[j] ]);
    }
  }
  if (ids.length > 0) {
    // load data from server about missing reactions
    ServerConnector.retreiveLightReactions(ids);
  }
  if (!ServerConnector.isWaitingForData()) {
    // if we already have everything then just refresh data to be visualized
    this.refreshSelectedLayouts();
    // and close "loading" dialog (if opened)
    GuiConnector.closeLoadingDialog();
  }
};

/**
 * This method checks if the layout contains any overlays (like AliasOverlay or
 * ReactionOverlay) that is currently visible on the map.
 * 
 * @param layoutId
 *          identifier of the layout
 * @returns {Boolean}: <code>true</code> if the layout contains overlays to
 *          visualize, <code>false</code> otherwise
 */
CustomMap.prototype.layoutContainsOverlays = function(layoutId) {

  // first, check top map
  if (this.selectedLayoutOverlays.hasOwnProperty(layoutId) && this.selectedLayoutOverlays[layoutId].length > 0) {
    return true;
  }

  // now check all submaps
  for (var i = 0; i < this.submaps.length; i++) {
    if (this.submaps[i].initialized) {
      if (this.submaps[i].selectedLayoutOverlays.hasOwnProperty(layoutId)
          && this.submaps[i].selectedLayoutOverlays[layoutId].length > 0) {
        return true;
      }
    }
  }
  return false;
};

/**
 * Refresh content of all {@link AliasInfoWindow} in this map and all submaps.
 */
CustomMap.prototype.refreshInfoWindows = function() {
  this._refreshInfoWindows();
  // now check all submaps
  for (var i = 0; i < this.submaps.length; i++) {
    this.submaps[i]._refreshInfoWindows();
  }
};

/**
 * Opens {@link AliasInfoWindow} for an {@link Alias} in a given model/submodel.
 * 
 * @param aliasId
 *          identifier of {@link Alias}
 * @param modelId
 *          identifier of {@link AbstractCustomMap}
 */
CustomMap.prototype.openInfoWindowForAlias = function(aliasId, modelId) {
  logger.debug("Opening info window for alias: " + aliasId + ", model: " + modelId);
  var model = this.getSubmodelById(modelId);
  return model.getModel().getAliasById(aliasId, true).then(function() {
    model._openInfoWindowForAlias(aliasId);
  });
};

/**
 * Renders markers, lines, etc. for elements highlighted in OverlayCollection.
 * 
 * @param overlayCollection
 *          {@link OverlayCollection} to be processed
 * @param fitBounds
 *          <code>true</code> if the borders should fit bounds after creating
 *          all elements
 */
CustomMap.prototype.renderOverlayCollection = function(overlayCollection, fitBounds) {
  var self = this;
  if (overlayCollection instanceof OverlayCollection) {
    logger.warn("Deprecated method use: renderOverlayCollection(overlayCollection)");
  } else {
    fitBounds = overlayCollection.fitBounds;
    overlayCollection = overlayCollection.overlayCollection;
  }

  var elements = overlayCollection.getIdentifiedElements();

  var boundsArray = [];
  boundsArray[this.getId()] = new google.maps.LatLngBounds();
  for (var j = 0; j < this.submaps.length; j++) {
    boundsArray[this.submaps[j].getId()] = new google.maps.LatLngBounds();
  }

  var bounds;

  return Promise.each(
      elements,
      function(element) {
        var model = self.getSubmodelById(element.modelId);
        if (element.getType() === "ALIAS") {
          if (overlayCollection.aliasMarkers[element.getId()] !== undefined) {
            logger.warn("More than one marker in " + overlayCollection.name + " for alias " + element.getId()
                + ". Skipping duplicates.");
            return null;
          } else {
            return model.getModel().getAliasById(element.getId()).then(function(aliasData) {
              var aliasMarker = new AliasMarker(element.getId(), element.icon, aliasData, model);
              overlayCollection.aliasMarkers[element.getId()] = aliasMarker;
              bounds = aliasMarker.getBounds();
              boundsArray[element.getModelId()].extend(bounds.getNorthEast());
              boundsArray[element.getModelId()].extend(bounds.getSouthWest());
              return aliasMarker;
            });
          }
        } else if (element.getType() === "REACTION") {
          return model.getModel().getReactionById(element.getId()).then(function(reactionData) {
            var marker = null;
            var icon = element.getIcon();

            if (icon === null || icon === undefined) {
              // this should happen when we visualize search data (there is
              // no marker, but only flat overlay of the reaction lines)
              //
              marker = new ReactionOverlay(null, reactionData, model, false, element.getId());
            } else {
              // when we have icon defined (for instance when it comes from
              // comment) then we don't want to have overlayed reaction lines
              // but icon marker
              marker = new ReactionMarker(element.getId(), icon, reactionData, model);
            }
            overlayCollection.reactionMarkers[element.getId()] = marker;
            bounds = marker.getBounds();
            boundsArray[element.getModelId()].extend(bounds.getNorthEast());
            boundsArray[element.getModelId()].extend(bounds.getSouthWest());
            return marker;
          });
        } else if (element.getType() === "POINT") {
          var pointData = model.getModel().getPointDataByPoint(element.getPoint());
          var pointMarker = new PointMarker(pointData, element.icon, model);
          overlayCollection.pointMarkers[pointData.getId()] = pointMarker;
          bounds = pointMarker.getBounds();
          boundsArray[element.getModelId()].extend(bounds.getNorthEast());
          boundsArray[element.getModelId()].extend(bounds.getSouthWest());
          return pointMarker;
        } else {
          throw new Error("Unknown type of the element in overlay: " + element.type);
        }
      }).then(function() {
    for (var i = 0; i < elements.length; i++) {
      var element = elements[i];
      var infoWindow = self.getInfoWindowForIdentifiedElement(element);
      if (infoWindow !== null && infoWindow !== undefined) {
        self.retrieveOverlayDetailDataForElement(element, infoWindow.getOverlayFullViewArray());
        self.updateInfoWindowForIdentifiedElement(element);
      }
    }

    if (elements.length > 0 && fitBounds) {
      for ( var mapId in boundsArray) {
        if (boundsArray.hasOwnProperty(mapId)) {
          var map = self.getSubmodelById(mapId).getGoogleMap();
          bounds = boundsArray[mapId];
          if (map !== undefined && !bounds.isEmpty()) {
            if (typeof map.fitBounds2 !== "undefined") {
              map.fitBounds2(bounds);
            } else {
              map.fitBounds(bounds);
            }
          }
        }
      }
    }

  });

};

/**
 * Creates and register listeners to be called on events:
 * <ul>
 * <li>onAddAliases</li>
 * <li>onAddReactions</li>
 * </ul>
 */
CustomMap.prototype.createClientServerListeners = function() {
  this.registerListenerType("onAddAliases");
  this.registerListenerType("onAddReactions");

  var refreshLayoutsFun = function(e) {
    var self = e.object;
    if (!ServerConnector.isWaitingForData()) {
      self.refreshSelectedLayouts();
      // and close "loading" dialog (if opened)
      GuiConnector.closeLoadingDialog();
    }
  };

  var refreshOverlaysFun = function(e) {
    e.object.refreshMarkers();
  };

  this.addListener("onAddAliases", refreshLayoutsFun);
  this.addListener("onAddAliases", refreshOverlaysFun);

  this.addListener("onAddReactions", refreshLayoutsFun);
  this.addListener("onAddReactions", refreshOverlaysFun);

};

/**
 * Opens {@link AbstractInfoWindow} for a marker.
 * 
 * @param marker
 *          {@link AbstractMarker} for which info window should be opened
 */
CustomMap.prototype.openInfoWindowForMarker = function(marker) {
  var modelId = marker.getCustomMap().getId();
  var markerId = marker.getId();
  var model = this.getSubmodelById(modelId);
  logger.debug("Opening info window for " + marker.constructor.name + ": " + markerId + ", model: " + modelId);
  var elementType = marker.getType();

  // open AliasInfoWindow in a right model
  model._openInfoWindowForMarker(marker);

  var infoWindow = model.returnInfoWindowForMarker(marker);

  var element = new IdentifiedElement({
    objectId : markerId,
    modelId : modelId,
    type : elementType
  });

  this.retrieveOverlayDetailDataForElement(element, infoWindow.getOverlayFullViewArray());

};

/**
 * Sends requestes to download detailed data about element in all
 * {@link OverlayCollection}.
 * 
 * @param element
 *          element for which we want to have detailed information
 */
CustomMap.prototype.retrieveOverlayDetailDataForElement = function(element, general) {
  var promises = [];
  if (general === undefined) {
    logger.warn("general param is undefined!");
    general = [];
  }
  for ( var overlayName in this.overlayCollections) {
    if (this.overlayCollections.hasOwnProperty(overlayName)) {
      var overlay = this.overlayCollections[overlayName];

      var generalRequest = general[overlayName];
      if (generalRequest === undefined) {
        logger.warn("No information about general overlay request for overlay: ", overlayName);
        generalRequest = false;
      }
      generalRequest = generalRequest || !overlay.allowSearchById();

      if (overlay.constructor.name === "OverlayCollection") {
        if (overlay.allowGeneralSearch() || overlay.allowSearchById()) {
          if (overlay.isMissingDetailData(element, generalRequest)) {
            ServerConnector.sendOverlayDetailDataRequest(overlayName, element, generalRequest);
          }
        }
      } else {
        promises.push(overlay.getDetailDataByIdentifiedElement(element, generalRequest));
      }
    }
  }
  return Promise.all(promises);
};

/**
 * Updates info window identified by element given as a parameter.
 * 
 * @param identifiedElement
 *          element for which info window should be updated
 */
CustomMap.prototype.updateInfoWindowForIdentifiedElement = function(identifiedElement) {
  var infoWindow = this.getInfoWindowForIdentifiedElement(identifiedElement);
  if (infoWindow === null || infoWindow === undefined) {
    return;
  } else {
    infoWindow.update();
  }
};

/**
 * Returns data from all {@link OverlayCollection} for a given alias.
 * 
 * @param alias
 *          {@link Alias} for which overlay data will be returned
 * @param general
 *          if true then all elements will be returned, if false then only ones
 *          availble right now in the overlay
 * @returns data from all {@link OverlayCollection} for a given alias
 */
CustomMap.prototype.getOverlayDataForAlias = function(alias, general) {
  var identifiedElement = new IdentifiedElement(alias);
  return this.getOverlayDataForIdentifiedElement(identifiedElement, general);
};

/**
 * Returns data from all {@link OverlayCollection} for a given reaction.
 * 
 * @param reaction
 *          {@link Reaction} for which overlay data will be returned
 * @param general
 *          if true then all elements will be returned, if false then only ones
 *          availble right now in the overlay
 * @returns data from all {@link OverlayCollection} for a given alias
 */
CustomMap.prototype.getOverlayDataForReaction = function(reaction, general) {
  var identifiedElement = new IdentifiedElement(reaction);
  return this.getOverlayDataForIdentifiedElement(identifiedElement, general);
};

/**
 * Returns data from all {@link OverlayCollection} for a given {@link PointData}
 * 
 * @param point
 *          {@link PointData} for which overlay data will be returned
 * @returns data from all {@link OverlayCollection} for a given
 *          {@link PointData}
 */
CustomMap.prototype.getOverlayDataForPoint = function(point, general) {
  var identifiedElement = new IdentifiedElement(point);
  return this.getOverlayDataForIdentifiedElement(identifiedElement, general);
};

/**
 * Returns data from all {@link OverlayCollection} for element identified by the
 * parameter
 * 
 * @param identifiedElement
 *          {@link IdentifiedElement} for which overlay data will be returned
 * @returns data from all {@link OverlayCollection} for a given
 *          {@link IdentifiedElement}
 */
CustomMap.prototype.getOverlayDataForIdentifiedElement = function(identifiedElement, general) {
  if (general === undefined) {
    logger.warn("general parameter must be defined");
    general = [];
  }
  var promises = [];
  var overlays = [];
  for ( var overlayName in this.overlayCollections) {
    if (this.overlayCollections.hasOwnProperty(overlayName)) {
      var overlay = this.overlayCollections[overlayName];
      if (overlay.allowGeneralSearch() || overlay.allowSearchById()) {
        var generalFlag = general[overlay.getName()];
        if (generalFlag === undefined) {
          logger.warn("General flag for overlay: " + overlay.getName() + " is not defined, assuming false");
          generalFlag = false;
        }
        overlays.push(overlay);
        promises.push(overlay.getDetailDataByIdentifiedElement(identifiedElement, !overlay.allowSearchById() || generalFlag));
      }
    }
  }
  return new Promise(function(resolve,reject){
    Promise.all(promises).then(function(values){
      var result = [];
      for (var i=0;i<values.length;i++){
        result.push({overlay:overlays[i], data:values[i]});
      }
      resolve(result);
    }).catch(reject);
  });
};

/**
 * Returns {@link AbstractInfoWindow} for element identified by the parameter.
 * 
 * @param identifiedElement
 *          {@link IdentifiedElement} that determines for which element we want
 *          {@link AbstractInfoWindow}
 * @returns {@link AbstractInfoWindow} for element identified by the parameter
 */
CustomMap.prototype.getInfoWindowForIdentifiedElement = function(identifiedElement) {
  var model = this.getSubmodelById(identifiedElement.modelId);
  var infoWindow = null;
  if (identifiedElement.type === "ALIAS") {
    infoWindow = model.getAliasInfoWindowById(identifiedElement.getId());
  } else if (identifiedElement.type === "POINT") {
    infoWindow = model.getPointInfoWindowById(identifiedElement.getId());
  } else if (identifiedElement.type === "REACTION") {
    infoWindow = model.getReactionInfoWindowById(identifiedElement.getId());
  } else {
    throw new Error("Unknown type of IdentifiedElement: ", identifiedElement);
  }
  return infoWindow;
};

CustomMap.prototype.getActiveSubmapId = function() {
  return this._activeSubmapId;
};

CustomMap.prototype.setActiveSubmapId = function(submapId) {
  this._activeSubmapId = submapId;
};

CustomMap.prototype.setActiveSubmapClickCoordinates = function(coordinates) {
  if (!(coordinates instanceof google.maps.Point)) {
    throw new Error("Coordinates must be provided as google.maps.Point object, but found: "+coordinates );
  }
  this._activeSubmapCoordinates = coordinates;
};
CustomMap.prototype.getActiveSubmapClickCoordinates = function() {
  return this._activeSubmapCoordinates;
};

CustomMap.prototype.updateAliasesForLayout = function(layoutId, jsonAliases) {
  logger.debug("Updating aliases for layout: " + layoutId);

  // init layout data
  if (this.getModel().getLayoutDataById(layoutId) === undefined || this.getModel().getLayoutDataById(layoutId) === null) {
    this.getModel().initLayoutData(layoutId);
    for (var j = 0; j < this.submaps.length; j++) {
      this.submaps[j].getModel().initLayoutData(layoutId);
    }
  }

  var aliases = JSON.parse(jsonAliases);
  for (var i = 0; i < aliases.length; i++) {
    var alias = aliases[i];
    var model = this.getSubmodelById(alias.modelId);
    if (model !== null) {
      model.getModel().updateAliasForLayout(layoutId, alias);
      model.getAliasInfoWindowById(alias.idObject).update();
    } else {
      logger.warn("Unknown model: " + alias.modelId);
    }
  }

  this.retrieveMissingAliases();
};

CustomMap.prototype.getReferenceGenome = function(type, version) {
  if (this._referenceGenome[type] === undefined) {
    this._referenceGenome[type] = [];
  }
  if (this._referenceGenome[type][version] === undefined) {
    ServerConnector.sendReferenceGenomeDetailRequest(type, version);
    this._referenceGenome[type][version] = new ReferenceGenome(null);
    return null;
  } else {
    return this._referenceGenome[type][version];
  }
};

CustomMap.prototype.updateReferenceGenome = function(type, version, jsonObj) {
  if (this._referenceGenome[type] === undefined) {
    this._referenceGenome[type] = [];
  }
  this._referenceGenome[type][version] = new ReferenceGenome(jsonObj);
  this.refreshInfoWindows();
};

CustomMap.prototype.getTopMap = function() {
  return this;
};

CustomMap.prototype.setProject = function(project) {
  this._project = project;
};
CustomMap.prototype.getProject = function() {
  return this._project;
};

CustomMap.prototype.getTopOverviewImage = function() {
  return this.getProject().getTopOverviewImage();
};

CustomMap.prototype.addControl = function(element) {
  if (this._controls[element.id] !== undefined) {
    throw new Error("Element with id "+element.id+" already added to controls");
  }
  if (ControlType[element.id]===undefined) {
    throw new Error("Unknown control type: "+element.id);
  }
  this._controls[element.id]= element;
};

CustomMap.prototype.getControl = function(type) {
  if (ControlType[type]===undefined) {
    throw new Error("Unknown control type: " + type);
  }
  return this._controls[type];
};

CustomMap.prototype.setCommentDialog = function(commentDialog) {
  this._commentDialog = commentDialog;
  commentDialog.close = function(){
    jsfCommentDialog.hide();
  }
};

CustomMap.prototype.getCommentDialog = function() {
  return this._commentDialog;
};

CustomMap.prototype.openCommentDialog = function() {
  var self = this;
  return ServerConnector.getClosestElementsByCoordinates({modelId:this.getActiveSubmapId(), coordinates:this.getActiveSubmapClickCoordinates()}).then(function(elements){
    return self.getCommentDialog().open(elements);
  });
};

module.exports = CustomMap;
