"use strict";

var Alias = require('./Alias');
var Reaction = require('./Reaction');
var PointData = require('./PointData');

var logger = require('../../logger');

/**
 * This is object representing element tha should be visualized on the map. It's
 * very light and contains only the most important data. There are three types
 * of objects to visualize:
 * <ul>
 * <li>"ALIAS" - for {@link Alias} objects</li>
 * <li>"REACTION" - for {@link Reaction} objects</li>
 * <li>"POINT" - for any point on the map, the data connected to this kind of
 * objects are stored in {@link PointData}</li>
 * </ul>
 */
function IdentifiedElement(javaObject) {
  if (javaObject instanceof Alias) {
    this.setId(javaObject.getId());
    this.setModelId(javaObject.getModelId());
    this.type = "ALIAS";
  } else if (javaObject instanceof Reaction) {
    this.setId(javaObject.getId());
    this.setModelId(javaObject.getModelId());
    this.type = "REACTION";
  } else if (javaObject instanceof PointData) {
    this.setId(javaObject.getId());
    this.setModelId(javaObject.getModelId());
    this.type = "POINT";
  } else {
    // identifier of the object to visualize
    if (javaObject.objectId === undefined) {
      this.setId(javaObject.id);
    } else {
      this.setId(javaObject.objectId);
    }
    // which marker should be used to show this object
    this.icon = javaObject.icon;
    // on which model the element is located
    this.setModelId(javaObject.modelId);
    // what kind of object we are talking about
    this.type = javaObject.type;
  }

  if (this.type === undefined || this.type === null) {
    throw new Error("Type not defined for element: " + javaObject);
  }

  this.type = this.type.toUpperCase();
  if (this.type === "POINT") {
    var tmp = this.getId();
    if (tmp.indexOf("Point2D.Double") >= 0) {
      tmp = tmp.replace("Point2D.Double", "");
      tmp = JSON.parse(tmp);
    } else {
      tmp = tmp.replace("(", "");
      tmp = tmp.replace(")", "");
      tmp = tmp.split(",");
    }
    var x = parseFloat(tmp[0]).toFixed(2);
    var y = parseFloat(tmp[1]).toFixed(2);
    this._point = new google.maps.Point(x, y);
  } else if (this.type !== "ALIAS" && this.type !== "REACTION") {
    throw new Error("Unknown type of identified element: " + this.type);
  }

  if (this.getId() === undefined || this.getId() === null) {
    throw "Id not defined for element: " + javaObject;
  }
}

/**
 * Returns point where it should be visualized when the type of object is
 * "POINT".
 * 
 * @returns {@linke google.maps.Point} where it should be visualized when the
 *          type of object is "POINT".
 */
IdentifiedElement.prototype.getPoint = function() {
  if (this._point === undefined || this._point === null) {
    logger.warn("No point associated with IdentifiedElement");
    return null;
  } else {
    return this._point;
  }
};

/**
 * Returns element identifier.
 * 
 * @returns element identifier
 */
IdentifiedElement.prototype.getId = function() {
  return this.id;
};

IdentifiedElement.prototype.setId = function(id) {
  // some elements are identfied by id that is not a number (like point on the
  // map)
  if (!isNaN(id)) {
    id = parseInt(id);
  }
  this.id = id;
};

IdentifiedElement.prototype.setModelId = function(modelId) {
  if (modelId === undefined || modelId === null) {
    throw new Error("ModelId is invalid");
  }
  this.modelId = parseInt(modelId);
};

/**
 * Returns model identifier where element is placed.
 * 
 * @returns model identifier
 */
IdentifiedElement.prototype.getModelId = function() {
  return this.modelId;
};

/**
 * Returns type of the element. For now only three types are allowed:
 * <ul>
 * <li>"ALIAS" - for {@link Alias} objects</li>
 * <li>"REACTION" - for {@link Reaction} objects</li>
 * <li>"POINT" - for any point on the map, the data connected to this kind of
 * objects are stored in {@link PointData}</li>
 * </ul>
 * 
 * @returns type of the element
 */
IdentifiedElement.prototype.getType = function() {
  return this.type;
};

/**
 * Returns icon that should be used for visualization.
 * 
 * @returns icon that should be used for visualization
 */
IdentifiedElement.prototype.getIcon = function() {
  return this.icon;
};

module.exports = IdentifiedElement;
