"use strict";

var logger = require('../../logger');

var IdentifiedElement = require('../data/IdentifiedElement');

/**
 * This class is responsible for collecting and updating markers found by
 * different panels (ie. search, comments, drug, targets).
 * 
 * @param map
 *          element of type CustomMap that represents custom extension to google
 *          map
 * @param name
 *          name of the collection used to distinguish it from the other
 *          sources.
 */
function OverlayCollection(params) {
  // map, name, allowSearchById, allowGeneralSearch
  var self = this;

  if (params.map === undefined) {
    throw new Error("map param must be defined");
  }

  this.elements = [];
  this.setName(params.name);
  this.setMap(params.map);

  this._ids = [];

  this.setAllowSearchById(params.allowSearchById);
  this.setAllowGeneralSearch(params.allowGeneralSearch);

  this._detailDataByIdentifiedElement = [];

  // register source in the map
  this.getMap().registerSource(self);
}

/**
 * Returns true if overlay allows to get general data for element.
 */
OverlayCollection.prototype.allowGeneralSearch = function() {
  return this._allowGeneralSearch;
};

OverlayCollection.prototype.getName = function() {
  return this.name;
};

/**
 * Returns true if overlay allows to get data for element by search id.
 */
OverlayCollection.prototype.allowSearchById = function() {
  return this._allowSearchById;
};

/**
 * Method that should be called when new data from server appears. It updates
 * markers associated with the object.
 * 
 * @param newElements
 *          new elements that should be associated with the overlay
 * @param fitBounds
 *          should the bound on the map be fit after creating markers on the map
 *          representing newElements
 */
OverlayCollection.prototype.updateOverlays = function(newElements, fitBounds) {
  var self = this;
  this.elements = [];
  for (var i = 0; i < newElements.length; i++) {
    this.elements.push(new IdentifiedElement(newElements[i]));
  }
  this.fitBounds = fitBounds;
  this.getMap().updateOverlayCollection(self, fitBounds);
};

/**
 * Clears all elements.
 */
OverlayCollection.prototype.clear = function() {
  var self = this;
  this.elements = [];
  this.getMap().updateOverlayCollection(self);
};

/**
 * Sends request to server to refresh data.
 */
OverlayCollection.prototype.refresh = function() {
  ServerConnector.sendRefreshRequest(this.name);
};

OverlayCollection.prototype.setResultsIds = function(ids) {
  this._ids = ids;
  this.getMap().refreshInfoWindows();
};

OverlayCollection.prototype.getResultsIds = function() {
  return this._ids;
};

OverlayCollection.prototype.isMissingDetailData = function(element, general) {
  var data = this._getDetailArrayByIdentifiedElement(element);
  if (typeof general !== "boolean") {
    logger.warn("general parameter is invalid, assuming 'false': ", general);
  }
  if (general === true) {
    if (this.allowGeneralSearch()) {
      return data['__FULL__'] === undefined;
    } else {
      throw new Error("Cannot execute general search for overlay: ", this.getName());
    }
  } else if (this.allowSearchById()) {
    for (var i = 0; i < this._ids.length; i++) {
      if (data[this._ids[i]] === undefined) {
        return true;
      }
    }
    return false;
  } else {
    logger.warn("No data is available for overlay: ", this.getName());
    return false;
  }
};

OverlayCollection.prototype.getMissingDetailDataIds = function(element, general) {
  if (typeof general !== "boolean") {
    logger.warn("general parameter is invalid, assuming 'false': ", general);
  }
  var data = this._getDetailArrayByIdentifiedElement(element);

  var result = [];
  if (general === true) {
    if (this.allowGeneralSearch()) {
      if (data['__FULL__'] === undefined) {
        result.push(null);
      }
    } else {
      throw new Error("Cannot execute general search for overlay: ", this.getName());
    }
  } else if (this.allowSearchById()) {
    for (var i = 0; i < this._ids.length; i++) {
      if (data[this._ids[i]] === undefined) {
        result.push(this._ids[i]);
      }
    }
  } else {
    logger
        .warn("Nothing is missing, because overlay doesn't allow this kind of search. Overlay name: ", this.getName());
  }
  return result;
};

OverlayCollection.prototype.updateOverviewElementDetailData = function(javaObject, searchResultIdentifier, newData) {
  var element = new IdentifiedElement(javaObject);
  var data = this._getDetailArrayByIdentifiedElement(element);

  if (searchResultIdentifier === null || searchResultIdentifier === undefined || searchResultIdentifier === "") {
    data.push(newData);
  } else {
    data[searchResultIdentifier] = newData;
  }
  this.getMap().updateInfoWindowForIdentifiedElement(element);
};

OverlayCollection.prototype.getDetailDataByIdentifiedElement = function(element, general) {
  if (typeof general !== "boolean") {
    logger.warn("general parameter is invalid, assuming 'false': ", general);
  }
  var data = this._getDetailArrayByIdentifiedElement(element);
  if (general === true) {
    if (this.allowGeneralSearch()) {
      return data;
    } else {
      logger.warn("No general data type for overlay: " + this.getName());
      return [];
    }
  } else {
    var result = [];
    for (var i = 0; i < this.getResultsIds().length; i++) {
      var id = this.getResultsIds()[i];
      var rowRes = data[id];
      if (rowRes !== undefined) {
        result[id] = rowRes;
      }
    }
    return result;
  }
};

OverlayCollection.prototype._getDetailArrayByIdentifiedElement = function(element) {
  var elementKey = element.getId() + "," + element.getType() + "," + element.getModelId();
  if (this._detailDataByIdentifiedElement[elementKey] === undefined) {
    this._detailDataByIdentifiedElement[elementKey] = [];
  }
  return this._detailDataByIdentifiedElement[elementKey];
};

OverlayCollection.prototype.setMap = function(map) {
  this._map = map;
};

OverlayCollection.prototype.getMap = function() {
  return this._map;
};

OverlayCollection.prototype.setName = function(name) {
  this.name = name;
};

OverlayCollection.prototype.getName = function() {
  return this.name;
};

OverlayCollection.prototype.setAllowSearchById = function(allowSearchById) {
  // configure if the overlay can contain detailed data about elements that
  // should be visualized in detailed mode of the Info Window
  if (typeof allowSearchById === "boolean") {
    this._allowSearchById = allowSearchById;
  } else if (allowSearchById === undefined) {
    this._allowSearchById = false;
  } else {
    throw new Error("Unknown type of allowSearchById: ", allowSearchById);
  }
};

OverlayCollection.prototype.setAllowGeneralSearch = function(allowGeneralSearch) {
  if (typeof allowGeneralSearch === "boolean") {
    this._allowGeneralSearch = allowGeneralSearch;
  } else if (allowGeneralSearch === undefined) {
    this._allowGeneralSearch = false;
  } else {
    throw new Error("Unknown type of allowSearchById: ", allowGeneralSearch);
  }
};

OverlayCollection.prototype.getIdentifiedElements = function(){
  return this.elements;
}

module.exports = OverlayCollection;
