"use strict";

var functions = require('../../../../main/js/Functions');

var Helper = require('../../Helper');

var Alias = require('../../../../main/js/map/data/Alias');
var AliasInfoWindow = require('../../../../main/js/map/window/AliasInfoWindow');
var IdentifiedElement = require('../../../../main/js/map/data/IdentifiedElement');
var LayoutAlias = require('../../../../main/js/map/data/LayoutAlias');
var OverlayCollection = require('../../../../main/js/map/overlay/OverlayCollection');

var assert = require('assert');

var logger = require('../../logger');

describe('AliasInfoWindow', function() {
  var helper;
  before(function() {
    helper = new Helper();
  });

  it("AliasInfoWindow Constructor", function() {
    var map = helper.createCustomMap();

    var alias = helper.createAlias();
    alias.setId(30001);
    alias.setIsComplete(false);
    alias.setModelId(map.getId());
    map.getModel().addAlias(alias);

    var aliasWindow = new AliasInfoWindow(alias, map);

    assert.equal(alias, aliasWindow.getAlias());
    assert.equal(map, aliasWindow.getCustomMap());
    assert.ok(aliasWindow.getContent().innerHTML.indexOf("loading") >= 0);
  });

  it("AliasInfoWindow Constructor 2", function() {
    var map = helper.createCustomMap();
    map.getOverlayDataForAlias = function() {
      return Promise.resolve([]);
    };

    var javaObject = {
      bounds : {
        x : 190,
        y : 44,
        width : 80,
        height : 40
      },
      modelId : map.getId(),
      idObject : 30001
    };
    var alias = new Alias(javaObject);
    map.getModel().addAlias(alias);

    var aliasWindow = new AliasInfoWindow(alias, map);

    assert.equal(alias, aliasWindow.alias);
    assert.equal(map, aliasWindow.customMap);
    assert.ok(aliasWindow.getContent().innerHTML.indexOf("loading") > -1);
  });

  it("Constructor 3", function(done) {
    var map = helper.createCustomMap();
    map.getTopMap = function() {
      var mockObj = {};
      mockObj.getOverlayDataForAlias = function() {
        return Promise.resolve([]);
      };
      mockObj.getSelectedLayouts = function() {
        var result = [];
        result.push(573);
        return result;
      };
      return mockObj;
    };

    // create a mock function that will return list of layouts alias
    // data
    map.getAliasVisibleLayoutsData = function() {
      return new Promise(function(resolve) {
        var result = [];
        result.push(new LayoutAlias({
          idObject : "1",
          color : {
            falpha : 0,
            value : -8388737
          }
        }));
        result.push(null);
        resolve(result);
      });
    };

    map.getVisibleLayoutNames = function() {
      return [ "name 1", "name 2" ];
    };

    var javaObject = {
      notes : "",
      type : "Protein",
      name : "s1",
      synonyms : [],
      formerSymbols : [],
      references : [],
      other : [],
      bounds : {
        x : 59,
        y : 73,
        width : 80,
        height : 40
      },
      modelId : map.getId(),
      idObject : 18552
    };
    var alias = new Alias(javaObject);
    map.getModel().addAlias(alias);

    var onload = function(){
      assert.equal(alias, aliasWindow.alias);
      assert.equal(map, aliasWindow.customMap);
      assert.ok(aliasWindow.getContent().innerHTML.indexOf("loading") === -1);
      done();
    }
    var aliasWindow = new AliasInfoWindow(alias, map, onload);

  });

  it("AliasInfoWindow update test", function() {
    var map = helper.createCustomMap();

    map.getTopMap = function() {
      var mockObj = {};
      mockObj.getOverlayDataForAlias = function() {
        return Promise.resolve([]);
      };
      mockObj.getSelectedLayouts = function() {
        var result = [];
        result.push(444);
        return result;
      };
      return mockObj;
    };

    // create a mock function that will return list of layouts alias
    // data
    map.getAliasVisibleLayoutsData = function() {
      return new Promise(function(resolve) {
        var result = [];
        result.push(new LayoutAlias({
          idObject : "1",
          color : {
            falpha : 0,
            value : -8388737
          }
        }));
        result.push(null);
        resolve(result);
      });
    };
    map.getVisibleLayoutNames = function() {
      return [ "name 1", "name 2" ];
    };

    var alias = helper.createAlias(map);
    alias.setIsComplete(true);

    var aliasWindow = new AliasInfoWindow(alias, map);

    var jsonString = '{"notes":"","type":"Protein","name":"s1","synonyms":[],'
        + '"formerSymbols":[],"references":[],"other":[],'
        + '"bounds":{"x":59,"y":73,"width":80,"height":40},"modelId":54,"idObject":18552}';
    var javaObject = JSON.parse(jsonString);
    alias.update(javaObject);

    aliasWindow.update();
    assert.ok(aliasWindow.getContent().innerHTML.indexOf("loading") === -1);
  });

  it("AliasInfoWindow createOverlayInfoDiv", function() {
    var map = helper.createCustomMap();

    var oc = new OverlayCollection({
      map : map,
      name : 'test name',
      allowSearchById : false,
      allowGeneralSearch : true
    });

    var alias = helper.createAlias(map);
    var aliasWindow = new AliasInfoWindow(alias, map);

    var data = oc.getDetailDataByIdentifiedElement(new IdentifiedElement(alias), true);
    data['test'] = {
      xField : 'y_param'
    };

    var overlayDiv = aliasWindow.createOverlayInfoDiv(oc, data);

    assert.ok(functions.isDomElement(overlayDiv));
    assert.ok(overlayDiv.innerHTML.indexOf('xField') >= 0);

  });

  it("AliasInfoWindow createDrugOverlayInfoDiv", function() {
    var map = helper.createCustomMap();

    var oc = helper.createDrugDbOverlay(map);

    var alias = helper.createAlias(map);
    var aliasWindow = new AliasInfoWindow(alias, map);

    var data = oc.getDetailDataByIdentifiedElement(new IdentifiedElement(alias), true);

    var emptyOverlayDiv = aliasWindow.createOverlayInfoDiv(oc, data);

    // check if we have row with info about empty drug list
    assert.ok(emptyOverlayDiv.innerHTML.indexOf("colspan") >= 0);

    data['test drug'] = {
      databaseSource : {},
      references : [ {} ]
    };

    var overlayDiv = aliasWindow.createOverlayInfoDiv(oc, data);

    assert.ok(functions.isDomElement(overlayDiv));
    assert.ok(overlayDiv.innerHTML.indexOf('test drug') >= 0);

  });

  it("AliasInfoWindow createCommentOverlayInfoDiv", function() {
    var map = helper.createCustomMap();

    var oc = helper.createCommentDbOverlay(map);

    var alias = helper.createAlias();
    alias.setId(3001);
    alias.setIsComplete(true);
    alias.setModelId(map.getId());
    map.getModel().addAlias(alias);
    

    var aliasWindow = new AliasInfoWindow(alias, map);

    return oc.getDetailDataByIdentifiedElement(new IdentifiedElement(alias), true).then(function(data){
      var comment = helper.createComment(alias);
      comment.setContent("test comment Content");    
      
      data[0] = comment; 
      data['__FULL__'] = null;
      
      var overlayDiv = aliasWindow.createOverlayInfoDiv(oc, data);

      assert.ok(functions.isDomElement(overlayDiv));
      assert.ok(overlayDiv.innerHTML.indexOf(comment.getContent()) >= 0);
      assert.ok(overlayDiv.innerHTML.indexOf(comment.getId()) >= 0);
    });


  });

  it("AliasInfoWindow createWaitingContentDiv", function() {
    var map = helper.createCustomMap();
    var alias = helper.createAlias(map);
    alias.setIsComplete(true);

    var aliasWindow = new AliasInfoWindow(alias, map);

    assert.ok(functions.isDomElement(aliasWindow.createWaitingContentDiv()));
  });
});
